#define SERVER_DEAD 0
#define SERVER_STARTING 1	/* Server Starting up */
#define SERVER_READY 2		/* Waiting for connection (or accept() lock) */
#define SERVER_BUSY_READ 3	/* Reading a client request */
#define SERVER_BUSY_WRITE 4	/* Processing a client request */
#define SERVER_BUSY_KEEPALIVE 5	/* Waiting for more requests via keepalive */
#define SERVER_BUSY_LOG 6	/* Logging the request */
#define SERVER_BUSY_DNS 7	/* Looking up a hostname */
#define SERVER_CLOSING 8	/* Closing the connection */
#define SERVER_GRACEFUL 9	/* server is gracefully finishing request */
#define SERVER_IDLE_KILL 10     /* Server is cleaning up idle children. */
#define SERVER_NUM_STATUS 11	/* number of status settings */

/* Type used for generation indicies.  Startup and every restart cause a
 * new generation of children to be spawned.  Children within the same
 * generation share the same configuration information -- pointers to stuff
 * created at config time in the parent are valid across children.  However,
 * this can't work effectively with non-forked architectures.  So while the
 * arrays in the scoreboard never change between the parent and forked
 * children, so they do not require shm storage, the contents of the shm
 * may contain no pointers.
 */
typedef int ap_generation_t;

/* Is the scoreboard shared between processes or not? 
 * Set by the MPM when the scoreboard is created.
 */
typedef enum {
    SB_NOT_SHARED = 1,
    SB_SHARED = 2,
    SB_DISPLAY = 3
} ap_scoreboard_e;

#define SB_DECLINED -1		/**< Module declines to handle */
#define SB_DONE -2			/**< Module has served the response completely 
				 *  - it's safe to die() with no more output
				 */
#define SB_OK 0			/**< Module has handled this stage. */

#define SB_WORKING  0  /* The server is busy and the child is useful. */
#define SB_IDLE_DIE 1  /* The server is idle and the child is superfluous. */
                       /*   The child should check for this and exit gracefully. */

/* stuff which is worker specific */
/***********************WARNING***************************************/
/* These are things that are used by mod_status. Do not put anything */
/*   in here that you cannot live without. This structure will not   */
/*   be available if mod_status is not loaded.                       */
/*********************************************************************/
typedef struct worker_score worker_score;

#ifndef _MAX_PATH
#define _MAX_PATH    255 /* maximum length of full pathname */
#define SCOREMAXPATHLEN _MAX_PATH*2
#endif
struct worker_score {
    int thread_num;
#if APR_HAS_THREADS
    apr_os_thread_t tid;
#endif
    int status;
    unsigned long access_count;
    off_t     bytes_served;
    unsigned long my_access_count;
    off_t     my_bytes_served;
    off_t     conn_bytes;
    unsigned short conn_count;
    time_t start_time;
    time_t stop_time;
#ifdef HAVE_TIMES
    struct tms times;
#endif
    time_t last_used;
    char op[5];				/* Keep 'em small... */
    char module[32];		/* Keep 'em small... */
    char client[32];		/* Keep 'em small... */
    char request[SCOREMAXPATHLEN];		/* We just want an idea... */
    char hostname[64];	        /* What virtual host is being accessed? */
    char hostip[32];	        /* What virtual host is being accessed? */

	int64 total_size;
	int64 total_transferred_size;
	int64 total_written;
	int64 total_read;
	int64 literal_data;
	int64 matched_data;
	int64 flist_size;
	int64 num_files;
	int64 num_transferred_files;
	int64 flist_index;
	char fname[SCOREMAXPATHLEN];
	char error[SCOREMAXPATHLEN*2];
	long thruput;
};

typedef struct {
    int             server_limit;
    int             thread_limit;
    ap_scoreboard_e sb_type;
    ap_generation_t running_generation;	/* the generation of children which
                                         * should still be serving requests. */
    time_t restart_time;
} global_score;

/* stuff which the parent generally writes and the children rarely read */
typedef struct process_score process_score;
struct process_score{
    pid_t pid;
    ap_generation_t generation;	/* generation of this child */
    ap_scoreboard_e sb_type;
    int quiescing;          /* the process whose pid is stored above is
                             * going down gracefully
                             */
	int connections;                           
	int threads;
	int64 total_size;
	int64 total_transferred_size;
	int64 total_written;
	int64 total_read;
	int64 num_transferred_files;
};

/* Scoreboard is now in 'local' memory, since it isn't updated once created,
 * even in forked architectures.  Child created-processes (non-fork) will
 * set up these indicies into the (possibly relocated) shmem records.
 */
typedef struct {
    global_score *global;
    process_score *parent;
    worker_score **servers;
} scoreboard;

typedef struct ap_sb_handle_t ap_sb_handle_t;

int ap_exists_scoreboard_image(void);
void ap_increment_counts(ap_sb_handle_t *sbh, struct jni_stats* pStats);
void ap_set_request(ap_sb_handle_t *sbh, const char *module_name, const char*request, const char *auth_user, const char *host, const char *addr);

void ap_display_scoreboard (void);

void ap_increment_process_counts(int process_num, int numConnections, int numThreads);
void ap_decrement_process_counts(int process_num, int numConnections, int numThreads);
void ap_increment_process_stats(int process_num, struct stats* pStats);

int ap_create_scoreboard(ap_scoreboard_e t);
int ap_reopen_scoreboard(int detached);
void ap_init_scoreboard(void *shared_score);
int ap_calc_scoreboard_size(void);
int ap_cleanup_scoreboard(void *d);

int ap_update_child_status(ap_sb_handle_t *sbh, int status);
int ap_update_child_status_from_indexes(int child_num, int thread_num,
                                                    int status);
void ap_time_process_request(int child_num, int thread_num, int status);

ap_sb_handle_t * ap_create_sb_handle(int child_num, int threadid);

void ap_delete_sb_handle(ap_sb_handle_t * sb_handle);

worker_score * ap_get_scoreboard_worker(int x, int y);
process_score * ap_get_scoreboard_process(int x);
global_score * ap_get_scoreboard_global(void);

scoreboard * ap_get_scoreboard(void);
scoreboard * ap_get_scoreboard_copy(void);

//extern scoreboard *ap_scoreboard_image;
//extern const char *ap_scoreboard_fname;
extern int ap_extended_status;

extern ap_generation_t volatile ap_my_generation;
